@@ -1,15 +1,19 @@
+
 require "wsapi.request"
 require "wsapi.response"
 require "wsapi.util"
-require "orbit.model"
 
-module("orbit", package.seeall)
+local _G, setfenv = _G, setfenv
+module("orbit")
+local _M = _M
+setfenv(1, _G)
 
-_VERSION = "2.0.2"
-_COPYRIGHT = "Copyright (C) 2007 Kepler Project"
-_DESCRIPTION = "MVC Web Development for the Kepler platform"
+_M._NAME = "orbit"
+_M._VERSION = "2.1.0"
+_M._COPYRIGHT = "Copyright (C) 2007-2009 Kepler Project"
+_M._DESCRIPTION = "MVC Web Development for the Kepler platform"
 
-mime_types = {
+_M.mime_types = {
   ez = "application/andrew-inset",
   atom = "application/atom+xml",
   hqx = "application/mac-binhex40",
@@ -168,9 +172,11 @@ mime_types = {
   atom = "application/atom+xml"
 }
 
-app_module_methods = {}
+_M.app_module_methods = {}
+local app_module_methods = _M.app_module_methods
 
-web_methods = {}
+_M.web_methods = {}
+local web_methods = _M.web_methods
 
 local function flatten(t)
    local res = {}
@@ -184,7 +190,7 @@ local function flatten(t)
    return table.concat(res)
 end
 
-function make_tag(name, data, class)
+local function make_tag(name, data, class)
   if class then class = ' class="' .. class .. '"' else class = "" end
   if not data then
     return "<" .. name .. class .. "/>"
@@ -205,7 +211,7 @@ function make_tag(name, data, class)
   end      
 end
 
-function new(app_module)
+function _M.new(app_module)
    if type(app_module) == "string" then
       app_module = { _NAME = app_module }
    else
@@ -215,11 +221,10 @@ function new(app_module)
       app_module[k] = v
    end
    app_module.run = function (wsapi_env) 
-		       return run(app_module, wsapi_env)
+		       return _M.run(app_module, wsapi_env)
 		    end
-   local table_prefix = (app_module._NAME and app_module._NAME .. "_") or ""
-   app_module.real_path = wsapi.app_path
-   app_module.mapper = orbit.model.new(table_prefix)
+   app_module.real_path = wsapi.app_path or "."
+   app_module.mapper = { default = true }
    app_module.not_found = function (web)
 			     web.status = "404 Not Found"
 			     return [[<html>
@@ -232,7 +237,7 @@ function new(app_module)
 				      <head><title>Server Error</title></head>
 					 <body><pre>]] .. msg .. [[</pre></body></html>]]
 			     end
-   app_module.dispatch_table = { get = {}, post = {} }
+   app_module.dispatch_table = { get = {}, post = {}, put = {}, delete = {} }
    return app_module
 end
 
@@ -257,15 +262,28 @@ function app_module_methods.dispatch_post(app_module, func, ...)
    end
 end
 
-function app_module_methods.dispatch_wsapi(app_module, func, ...)
+function app_module_methods.dispatch_put(app_module, func, ...)
    for _, pat in ipairs{ ... } do
-      table.insert(app_module.dispatch_table.post, { pattern = pat, 
-		      handler = func, wsapi = true })
-      table.insert(app_module.dispatch_table.get, { pattern = pat, 
-		      handler = func, wsapi = true })
+      table.insert(app_module.dispatch_table.put, { pattern = pat, 
+		      handler = func })
    end
 end
 
+function app_module_methods.dispatch_delete(app_module, func, ...)
+   for _, pat in ipairs{ ... } do
+      table.insert(app_module.dispatch_table.delete, { pattern = pat, 
+		      handler = func })
+   end
+end
+
+function app_module_methods.dispatch_wsapi(app_module, func, ...)
+  for _, pat in ipairs{ ... } do
+    for _, tab in pairs(app_module.dispatch_table) do
+      table.insert(tab, { pattern = pat, handler = func, wsapi = true })
+    end
+  end
+end
+
 function app_module_methods.dispatch_static(app_module, ...)
    app_module:dispatch_get(serve_file(app_module), ...)
 end
@@ -273,7 +291,7 @@ end
 function app_module_methods.serve_static(app_module, web, filename)
    local ext = string.match(filename, "%.([^%.]+)$")
    if app_module.use_xsendfile then
-      web.headers["Content-Type"] = mime_types[ext] or 
+      web.headers["Content-Type"] = _M.mime_types[ext] or 
 	 "application/octet-stream"
       web.headers["X-Sendfile"] = filename
       return "xsendfile"
@@ -282,7 +300,7 @@ function app_module_methods.serve_static(app_module, web, filename)
       if not file then
 	 return app_module.not_found(web)
       else
-	 web.headers["Content-Type"] = mime_types[ext] or 
+	 web.headers["Content-Type"] = _M.mime_types[ext] or 
 	    "application/octet-stream"
 	 local contents = file:read("*a")
 	 file:close()
@@ -331,7 +349,7 @@ local function htmlify_func(func)
   setfenv(func, env)
 end
 
-function htmlify(app_module, ...)
+function _M.htmlify(app_module, ...)
    if type(app_module) == "function" then
       htmlify_func(app_module)
       for _, func in ipairs{...} do
@@ -354,9 +372,16 @@ function htmlify(app_module, ...)
    end
 end
 
-app_module_methods.htmlify = htmlify
+app_module_methods.htmlify = _M.htmlify
 
 function app_module_methods.model(app_module, ...)
+   if app_module.mapper.default then
+      local table_prefix = (app_module._NAME and app_module._NAME .. "_") or ""
+      if not orbit.model then
+	 require "orbit.model"
+      end
+      app_module.mapper = orbit.model.new(table_prefix, app_module.mapper.conn)
+   end
    return app_module.mapper:new(...)
 end
 
@@ -385,7 +410,7 @@ function web_methods:static_link(url)
   local prefix = self.prefix or ""
   local is_script = prefix:match("(%.%w+)$")
   if not is_script then return self:link(url) end
-  local vpath = prefix:match("(.*)/")
+  local vpath = self.path_translated:sub(#self.doc_root+1):match("(.*)/")
   return vpath .. url
 end
 
@@ -393,6 +418,10 @@ function web_methods:empty(s)
   return not s or string.match(s, "^%s*$")
 end
 
+function web_methods:content_type(s)
+  self.headers["Content-Type"] = s
+end
+
 function web_methods:page(name, env)
   if not orbit.pages then
     require "orbit.pages"
@@ -434,7 +463,12 @@ local function dispatcher(app_module, method, path)
       return app_module["handle_" .. method], {}
    else
       for _, item in ipairs(app_module.dispatch_table[method]) do
-	 local captures = { string.match(path, "^" .. item.pattern .. "$") }
+	 local captures
+	 if type(item.pattern) == "string" then
+	    captures = { string.match(path, "^" .. item.pattern .. "$") }
+	 else
+	    captures = { item.pattern:match(path) }
+	 end
 	 if #captures > 0 then
 	    for i = 1, #captures do
 	      captures[i] = wsapi.util.url_decode(captures[i])
@@ -477,7 +511,7 @@ local function make_web_object(app_module, wsapi_env)
   return web, res
 end
 
-function run(app_module, wsapi_env)
+function _M.run(app_module, wsapi_env)
   local handler, captures, wsapi_handler = dispatcher(app_module,
 						      string.lower(wsapi_env.REQUEST_METHOD),
 						      wsapi_env.PATH_INFO)
@@ -506,3 +540,5 @@ function run(app_module, wsapi_env)
   end
   return res:finish()
 end
+
+return _M
